@@ -24,35 +24,42 @@ include Identifiable.S with type t := t
    - [compare] compares identifiers by binding location
 *)
 
+val print_with_scope : Format.formatter -> t -> unit
+        (** Same as {!print} except that it will also add a "[n]" suffix
+            if the scope of the argument is [n]. *)
 
-val create: string -> t
+
+val create_scoped: scope:int -> string -> t
+val create_local: string -> t
 val create_persistent: string -> t
-val create_predef_exn: string -> t
+val create_predef: string -> t
+
 val rename: t -> t
+        (** Creates an identifier with the same name as the input, a fresh
+            stamp, and no scope.
+            @raise [Fatal_error] if called on a persistent / predef ident. *)
+
 val name: t -> string
-val with_name: t -> string -> t
 val unique_name: t -> string
 val unique_toplevel_name: t -> string
 val persistent: t -> bool
 val same: t -> t -> bool
-        (* Compare identifiers by binding location.
-           Two identifiers are the same either if they are both
-           non-persistent and have been created by the same call to
-           [new], or if they are both persistent and have the same
-           name. *)
+        (** Compare identifiers by binding location.
+            Two identifiers are the same either if they are both
+            non-persistent and have been created by the same call to
+            [create_*], or if they are both persistent and have the same
+            name. *)
+
 val compare: t -> t -> int
-val hide: t -> t
-        (* Return an identifier with same name as the given identifier,
-           but stamp different from any stamp returned by new.
-           When put in a 'a tbl, this identifier can only be looked
-           up by name. *)
 
 val global: t -> bool
-val is_predef_exn: t -> bool
+val is_predef: t -> bool
+
+val scope: t -> int
+
+val lowest_scope : int
+val highest_scope: int
 
-val binding_time: t -> int
-val current_time: unit -> int
-val set_current_time: int -> unit
 val reinit: unit -> unit
 
 type 'a tbl
@@ -66,7 +73,7 @@ val find_all: string -> 'a tbl -> (t * 'a) list
 val fold_name: (t -> 'a -> 'b -> 'b) -> 'a tbl -> 'b -> 'b
 val fold_all: (t -> 'a -> 'b -> 'b) -> 'a tbl -> 'b -> 'b
 val iter: (t -> 'a -> unit) -> 'a tbl -> unit
-
+val remove: t -> 'a tbl -> 'a tbl
 
 (* Idents for sharing keys *)
 
